@@ -40,7 +40,6 @@
 #include "lm-internals.h"
 #include "lm-parser.h"
 #include "lm-sha.h"
-#include "lm-queue.h"
 #include "lm-connection.h"
 
 #define IN_BUFFER_SIZE 1024
@@ -50,16 +49,6 @@ typedef struct {
 	LmMessageHandler  *handler;
 } HandlerData;
 
-typedef struct {
-	LmConnection    *connection;
-	
-	/* struct to save resolved address */
-	struct addrinfo *resolved_addrs;
-	struct addrinfo *current_addr;
-	int              fd;
-	GIOChannel           *io_channel;
-} LmConnectData;
-
 typedef struct {
 	GSource       source;
 	LmConnection *connection;
@@ -67,42 +56,47 @@ typedef struct {
 
 struct _LmConnection {
 	/* Parameters */
-	gchar      *server;
-	guint       port;
+	GMainContext *context;
+	gchar        *server;
+	gchar        *jid;
+	guint         port;
 
-	LmSSL      *ssl;
+	LmSSL        *ssl;
 
-	LmProxy    *proxy;
+	LmProxy      *proxy;
 	
-	LmParser   *parser;
-	gchar      *stream_id;
+	LmParser     *parser;
+	gchar        *stream_id;
 
-	GHashTable *id_handlers;
-	GSList     *handlers[LM_MESSAGE_TYPE_UNKNOWN];
+	GHashTable   *id_handlers;
+	GSList       *handlers[LM_MESSAGE_TYPE_UNKNOWN];
 
 	/* Communication */
-	GIOChannel *io_channel;
-	guint       io_watch_in;
-	guint       io_watch_err;
-	guint       io_watch_hup;
-	guint       fd;
+	GIOChannel   *io_channel;
+	guint         io_watch_in;
+	guint         io_watch_err;
+	guint         io_watch_hup;
+	guint         fd;
 	
-	guint       open_id;
-	LmCallback *open_cb;
+	guint         open_id;
+	LmCallback   *open_cb;
 
-	gboolean    cancel_open;
-	LmCallback *close_cb;
-	LmCallback *auth_cb;
-	LmCallback *register_cb;
+	gboolean      cancel_open;
+	LmCallback   *close_cb;
+	LmCallback   *auth_cb;
+	LmCallback   *register_cb;
 
-	LmCallback *disconnect_cb;
+	LmCallback   *disconnect_cb;
 
-	LmQueue    *incoming_messages;
-	GSource    *incoming_source;
+	GQueue       *incoming_messages;
+	GSource      *incoming_source;
 
 	LmConnectionState state;
 
-	gint        ref_count;
+	guint         keep_alive_rate;
+	guint         keep_alive_id;
+
+	gint          ref_count;
 };
 
 typedef enum {
@@ -170,7 +164,14 @@ static void      connection_signal_disconnect   (LmConnection *connection,
 						 LmDisconnectReason reason);
 
 static void     connection_do_connect           (LmConnectData *connect_data);
-
+static guint    connection_add_watch            (LmConnection  *connection,
+						 GIOChannel    *channel,
+						 GIOCondition   condition,
+						 GIOFunc        func,
+						 gpointer       user_data);
+static gboolean connection_send_keep_alive      (LmConnection  *connection);
+static void     connection_start_keep_alive     (LmConnection  *connection);
+static void     connection_stop_keep_alive      (LmConnection  *connection);
 
 static GSourceFuncs incoming_funcs = {
 	connection_incoming_prepare,
@@ -185,6 +186,7 @@ connection_free (LmConnection *connection)
 	int i;
 
 	g_free (connection->server);
+	g_free (connection->jid);
 
 	/* Unref handlers */
 	for (i = 0; i < LM_MESSAGE_TYPE_UNKNOWN; ++i) {
@@ -262,15 +264,19 @@ connection_new_message_cb (LmParser     *parser,
 		    _lm_message_type_to_string (lm_message_get_type (m)),
 		    lm_message_node_get_attribute (m->node, "from"));
 
-	lm_queue_push_tail (connection->incoming_messages, m);
+	g_queue_push_tail (connection->incoming_messages, m);
 }
 
-static gboolean
-connection_succeeded (LmConnectData *connect_data)
+gboolean
+_lm_connection_succeeded (LmConnectData *connect_data)
 {
-	LmConnection *connection = connect_data->connection;
+	LmConnection *connection;
 	LmMessage    *m;
 	GIOFlags      flags;
+	gchar        *server_from_jid;
+	gchar        *ch;
+
+	connection = connect_data->connection;
 
 	/* Need some way to report error/success */
 	if (connection->cancel_open) {
@@ -295,8 +301,9 @@ connection_succeeded (LmConnectData *connect_data)
 	g_io_channel_set_flags (connection->io_channel, flags, NULL);
 
 	if (connection->ssl) {
-		if (!_lm_ssl_begin (connection->ssl, connection->fd, 
-				    connection->server, NULL)) {
+		if (!_lm_ssl_begin (connection->ssl, connection->fd,
+				    connection->server,
+				    NULL)) {
 			shutdown (connection->fd, SHUT_RDWR);
 			close (connection->fd);
 			connection_do_close (connection);
@@ -313,20 +320,26 @@ connection_succeeded (LmConnectData *connect_data)
 	g_io_channel_set_flags (connection->io_channel,
 				flags & G_IO_FLAG_NONBLOCK, NULL);
 	
-	connection->io_watch_in = g_io_add_watch (connection->io_channel,
-						  G_IO_IN,
-						  (GIOFunc) connection_in_event,
-						  connection);
+	connection->io_watch_in = 
+		connection_add_watch (connection,
+				      connection->io_channel,
+				      G_IO_IN,
+				      (GIOFunc) connection_in_event,
+				      connection);
 	
-	connection->io_watch_err = g_io_add_watch (connection->io_channel, 
-						   G_IO_ERR,
-						   (GIOFunc) connection_error_event,
+	connection->io_watch_err = 
+		connection_add_watch (connection,
+				      connection->io_channel, 
+				      G_IO_ERR,
+				      (GIOFunc) connection_error_event,
 						   connection);
-	connection->io_watch_hup = g_io_add_watch (connection->io_channel,
-						   G_IO_HUP,
-						   (GIOFunc) connection_hup_event,
-						   connection);
-
+	connection->io_watch_hup = 
+		connection_add_watch (connection,
+				      connection->io_channel,
+				      G_IO_HUP,
+				      (GIOFunc) connection_hup_event,
+				      connection);
+	
 	if (!connection_send (connection, 
 			      "<?xml version='1.0' encoding='UTF-8'?>", -1,
 			      NULL)) {
@@ -334,7 +347,13 @@ connection_succeeded (LmConnectData *connect_data)
 		return FALSE;
 	}
 
-	m = lm_message_new (connection->server, LM_MESSAGE_TYPE_STREAM);
+	if (connection->jid != NULL && (ch = strchr (connection->jid, '@')) != NULL) {
+		server_from_jid = ch + 1;
+	} else {
+		server_from_jid = connection->server;
+	}
+
+	m = lm_message_new (server_from_jid, LM_MESSAGE_TYPE_STREAM);
 	lm_message_node_set_attributes (m->node,
 					"xmlns:stream", 
 					"http://etherx.jabber.org/streams",
@@ -355,8 +374,8 @@ connection_succeeded (LmConnectData *connect_data)
 	return FALSE;
 }
 
-static void 
-connection_failed_with_error (LmConnectData *connect_data, int error) 
+void 
+_lm_connection_failed_with_error (LmConnectData *connect_data, int error) 
 {
 	g_log (LM_LOG_DOMAIN, LM_LOG_LEVEL_NET,
 	       "Connection failed: %s (error %d)\n",
@@ -377,10 +396,10 @@ connection_failed_with_error (LmConnectData *connect_data, int error)
 	}
 }
 
-static void 
-connection_failed (LmConnectData *connect_data)
+void 
+_lm_connection_failed (LmConnectData *connect_data)
 {
-	connection_failed_with_error (connect_data,errno);
+	_lm_connection_failed_with_error (connect_data,errno);
 }
 	
 static gboolean 
@@ -400,16 +419,10 @@ connection_connect_cb (GIOChannel   *source,
 		/* get the real error from the socket */
 		getsockopt (connect_data->fd, SOL_SOCKET, SO_ERROR, 
 			    &error, &len);
-		connection_failed_with_error (connect_data, error);
+		_lm_connection_failed_with_error (connect_data, error);
 		return FALSE;
 	} else if (condition == G_IO_OUT) {
-		if (connect_data->connection->proxy) {
-			if (!_lm_proxy_negotiate (connection->proxy, connect_data->fd, connection->server, connection->port)) {
-				connection_failed (connect_data);
-				return FALSE;
-			}
-		}
-		connection_succeeded (connect_data);
+		_lm_connection_succeeded (connect_data);
 	} else {
 		g_assert_not_reached ();
 	}
@@ -454,7 +467,7 @@ connection_do_connect (LmConnectData *connect_data)
 		     addr->ai_protocol);
 	
 	if (fd < 0) {
-		connection_failed (connect_data);
+		_lm_connection_failed (connect_data);
 		return;
 	}
 
@@ -466,16 +479,82 @@ connection_do_connect (LmConnectData *connect_data)
 	
 	if (res < 0 && errno != EINPROGRESS) {
 		close (fd);
-		connection_failed (connect_data);
+		_lm_connection_failed (connect_data);
 		return;
 	}
 	
 	connect_data->io_channel = g_io_channel_unix_new (fd);
-	g_io_add_watch (connect_data->io_channel, G_IO_OUT|G_IO_ERR,
-			(GIOFunc) connection_connect_cb, connect_data);
+	if (connection->proxy) {
+		connection_add_watch (connection,
+				      connect_data->io_channel,
+				      G_IO_OUT|G_IO_ERR,
+				      (GIOFunc) _lm_proxy_connect_cb, 
+				      connect_data);
+	} else {
+		connection_add_watch (connection,
+				      connect_data->io_channel,
+				      G_IO_OUT|G_IO_ERR,
+				      (GIOFunc) connection_connect_cb,
+				      connect_data);
+	}
+
 	return;
 }
 
+static guint
+connection_add_watch (LmConnection *connection,
+		      GIOChannel   *channel,
+		      GIOCondition  condition,
+		      GIOFunc       func,
+		      gpointer      user_data)
+{
+	GSource *source;
+	guint    id;
+                                                                                
+	g_return_val_if_fail (channel != NULL, 0);
+                                                                                
+	source = g_io_create_watch (channel, condition);
+                                                                                
+	g_source_set_callback (source, (GSourceFunc)func, user_data, NULL);
+                                                                                
+	id = g_source_attach (source, connection->context);
+	g_source_unref (source);
+  
+	return id;
+}
+
+static gboolean
+connection_send_keep_alive (LmConnection *connection)
+{ 
+	if (!connection_send (connection, " ", -1, NULL)) {
+		lm_verbose ("Error while sending keep alive package");
+	}
+
+	return TRUE;
+}
+
+static void
+connection_start_keep_alive (LmConnection *connection)
+{
+	if (connection->keep_alive_id != 0) {
+		connection_stop_keep_alive (connection);
+	}
+
+	connection->keep_alive_id = g_timeout_add (connection->keep_alive_rate,
+						   (GSourceFunc) connection_send_keep_alive,
+						   connection);
+}
+
+static void
+connection_stop_keep_alive (LmConnection *connection)
+{
+	if (connection->keep_alive_id != 0) {
+		g_source_remove (connection->keep_alive_id);
+	}
+
+	connection->keep_alive_id = 0;
+}
+
 /* Returns directly */
 /* Setups all data needed to start the connection attempts */
 static gboolean
@@ -503,7 +582,7 @@ connection_do_open (LmConnection *connection, GError **error)
 
 	/* source thingie for messages and stuff */
 	connection->incoming_source = connection_create_source (connection);
-	g_source_attach (connection->incoming_source,NULL);
+	g_source_attach (connection->incoming_source, connection->context);
 	
 	lm_verbose ("Connecting to: %s:%d\n", 
 		    connection->server, connection->port);
@@ -515,7 +594,7 @@ connection_do_open (LmConnection *connection, GError **error)
 	req.ai_protocol = IPPROTO_TCP;
 	
 	connection->cancel_open = FALSE;
-	connection->state = LM_CONNECTION_STATE_CONNECTING;
+	connection->state = LM_CONNECTION_STATE_OPENING;
 	
 	if (connection->proxy) {
 		const gchar *proxy_server;
@@ -534,9 +613,11 @@ connection_do_open (LmConnection *connection, GError **error)
 		}
 	} else { /* connect directly */
 		g_log (LM_LOG_DOMAIN,LM_LOG_LEVEL_NET,
-		       "Going to connect to %s\n",connection->server);
+		       "Going to connect to %s\n", 
+		       connection->server);
 
-		if (getaddrinfo (connection->server, NULL, &req, &ans) != 0) {
+		if (getaddrinfo (connection->server,
+				 NULL, &req, &ans) != 0) {
 			g_set_error (error,
 				     LM_ERROR,                 
 				     LM_ERROR_CONNECTION_OPEN,   
@@ -565,6 +646,8 @@ connection_do_open (LmConnection *connection, GError **error)
 static void
 connection_do_close (LmConnection *connection)
 {
+	connection_stop_keep_alive (connection);
+
 	if (connection->io_channel) {
 		g_source_remove (connection->io_watch_in);
 		g_source_remove (connection->io_watch_err);
@@ -580,8 +663,8 @@ connection_do_close (LmConnection *connection)
 	if (!lm_connection_is_open (connection)) {
 		return;
 	}
-
-	connection->state = LM_CONNECTION_STATE_DISCONNECTED;
+	
+	connection->state = LM_CONNECTION_STATE_CLOSED;
 
 	if (connection->ssl) {
 		_lm_ssl_close (connection->ssl);
@@ -693,7 +776,7 @@ connection_send (LmConnection  *connection,
 {
 	gsize             bytes_written;
 	
-	if (connection->state < LM_CONNECTION_STATE_CONNECTING) {
+	if (connection->state < LM_CONNECTION_STATE_OPENING) {
 		g_set_error (error,
 			     LM_ERROR,
 			     LM_ERROR_CONNECTION_NOT_OPEN,
@@ -884,7 +967,7 @@ connection_auth_reply (LmMessageHandler *handler,
 	} 
 	else if (strcmp (type, "error") == 0) {
 		result = FALSE;
-		connection->state = LM_CONNECTION_STATE_CONNECTED;
+		connection->state = LM_CONNECTION_STATE_OPEN;
 	}
 	
 	lm_verbose ("AUTH reply: %d\n", result);
@@ -916,11 +999,13 @@ connection_stream_received (LmConnection *connection, LmMessage *m)
 	
 	lm_verbose ("Stream received: %s\n", connection->stream_id);
 	
-	connection->state = LM_CONNECTION_STATE_CONNECTED;
+	connection->state = LM_CONNECTION_STATE_OPEN;
 	
 	/* Check to see if the stream is correctly set up */
 	result = TRUE;
 
+	connection_start_keep_alive (connection);
+
 	if (connection->open_cb && connection->open_cb->func) {
 		LmCallback *cb = connection->open_cb;
 		
@@ -945,7 +1030,7 @@ connection_incoming_prepare (GSource *source, gint *timeout)
 	
 	connection = ((LmIncomingSource *)source)->connection;
 	
-	return !lm_queue_is_empty (connection->incoming_messages);
+	return !g_queue_is_empty (connection->incoming_messages);
 }
 
 static gboolean
@@ -964,7 +1049,7 @@ connection_incoming_dispatch (GSource *source,
 	
 	connection = ((LmIncomingSource *) source)->connection;
 
-	m = (LmMessage *) lm_queue_pop_head (connection->incoming_messages);
+	m = (LmMessage *) g_queue_pop_head (connection->incoming_messages);
 	
 	if (m) {
 		connection_handle_message (connection, m);
@@ -1022,14 +1107,17 @@ lm_connection_new (const gchar *server)
 	} else {
 		connection->server = NULL;
 	}
-	
+
+	connection->context           = NULL;
 	connection->port              = LM_CONNECTION_DEFAULT_PORT;
+	connection->jid               = NULL;
 	connection->ssl               = NULL;
 	connection->proxy             = NULL;
 	connection->disconnect_cb     = NULL;
-	connection->incoming_messages = lm_queue_new ();
+	connection->incoming_messages = g_queue_new ();
 	connection->cancel_open       = FALSE;
-	connection->state             = LM_CONNECTION_STATE_DISCONNECTED;
+	connection->state             = LM_CONNECTION_STATE_CLOSED;
+	connection->keep_alive_id     = 0;
 	
 	connection->id_handlers = g_hash_table_new_full (g_str_hash, 
 							 g_str_equal,
@@ -1048,6 +1136,29 @@ lm_connection_new (const gchar *server)
 	return connection;
 }
 
+/**
+ * lm_connection_new_with_context:
+ * @server: The hostname to the server for the connection.
+ * @context: The context this connection should be running in.
+ * 
+ * Creates a new closed connection running in a certain context. To open the 
+ * connection call #lm_connection_open. @server can be #NULL but must be set 
+ * before calling #lm_connection_open.
+ * 
+ * Return value: A newly created LmConnection, should be unreffed with lm_connection_unref().
+ **/
+LmConnection *
+lm_connection_new_with_context (const gchar *server, GMainContext *context)
+{
+	LmConnection *connection;
+
+	connection = lm_connection_new (server);
+	connection->context = context;
+
+	g_main_context_ref (connection->context);
+
+	return connection;
+}
 
 /**
  * lm_connection_open:
@@ -1100,19 +1211,16 @@ lm_connection_open_and_block (LmConnection *connection, GError **error)
 		return FALSE;
 	}
 	
-	while ((state = lm_connection_get_state (connection)) == LM_CONNECTION_STATE_CONNECTING) {
-		g_print ("while...\n");
-		if (g_main_context_pending (NULL)) {
-			g_print ("Iterating\n");
-			g_main_context_iteration (NULL, TRUE);
+	while ((state = lm_connection_get_state (connection)) == LM_CONNECTION_STATE_OPENING) {
+		if (g_main_context_pending (connection->context)) {
+			g_main_context_iteration (connection->context, TRUE);
 		} else {
 			usleep (10);
 		}
 	}
 
-	g_print ("Connected!\n");
-	
 	if (lm_connection_is_open (connection)) {
+		connection_start_keep_alive (connection);
 		return TRUE;
 	}
 
@@ -1123,7 +1231,7 @@ lm_connection_open_and_block (LmConnection *connection, GError **error)
  * lm_connection_cancel_open:
  * @connection: an #LmConnection to cancel opening on
  *
- * Cancels the open operation of a connection. The connection should be in the state #LM_CONNECTION_STATE_CONNECTING.
+ * Cancels the open operation of a connection. The connection should be in the state #LM_CONNECTION_STATE_OPENING.
  **/
 void
 lm_connection_cancel_open (LmConnection *connection)
@@ -1155,10 +1263,9 @@ lm_connection_close (LmConnection      *connection,
 			     "Connection is not open, call lm_connection_open() first");
 		return FALSE;
 	}
-	
+
 	lm_verbose ("Disconnecting from: %s:%d\n", 
-		    connection->server,
-		    connection->port);
+		    connection->server, connection->port);
 	
 	if (!connection_send (connection, "</stream:stream>", -1, error)) {
 		return FALSE;
@@ -1283,7 +1390,6 @@ lm_connection_authenticate_and_block (LmConnection  *connection,
 		return FALSE;
 	}
 
-	g_print ("Foo: %d\n", (gint)connection);
 	m = connection_create_auth_msg (connection,
 					username, 
 					password,
@@ -1318,6 +1424,33 @@ lm_connection_authenticate_and_block (LmConnection  *connection,
 	return FALSE;
 }
 
+/**
+ * lm_connection_set_keep_alive_rate:
+ * @connection: #LmConnection to check if it is open.
+ * @rate: Number of seconds between keep alive packages are sent.
+ * 
+ * Set the keep alive rate, in seconds. Set to 0 to prevent keep alive messages to be sent.
+ * A keep alive message is a single space character.
+ **/
+void
+lm_connection_set_keep_alive_rate (LmConnection *connection, guint rate)
+{
+	g_return_if_fail (connection != NULL);
+
+	connection_stop_keep_alive (connection);
+
+	if (rate == 0) {
+		connection->keep_alive_id = 0;
+		return;
+	}
+
+	connection->keep_alive_rate = rate * 1000;
+	
+	if (lm_connection_is_open (connection)) {
+		connection_start_keep_alive (connection);
+	}
+}
+
 /**
  * lm_connection_is_open:
  * @connection: #LmConnection to check if it is open.
@@ -1331,7 +1464,7 @@ lm_connection_is_open (LmConnection *connection)
 {
 	g_return_val_if_fail (connection != NULL, FALSE);
 	
-	return connection->state >= LM_CONNECTION_STATE_CONNECTED;
+	return connection->state >= LM_CONNECTION_STATE_OPEN;
 }
 
 /**
@@ -1354,7 +1487,7 @@ lm_connection_is_authenticated (LmConnection *connection)
  * lm_connection_get_server:
  * @connection: an #LmConnection
  * 
- * Fetches the server address that @connection is using.
+ * Fetches the server address that @connection is using. 
  * 
  * Return value: the server address
  **/
@@ -1371,7 +1504,8 @@ lm_connection_get_server (LmConnection *connection)
  * @connection: an #LmConnection
  * @server: Address of the server
  * 
- * Sets the server address for @connection to @server. Notice that @connection can't be open while doing this.
+ * Sets the server address for @connection to @server. Notice that @connection
+ * can't be open while doing this. 
  **/
 void
 lm_connection_set_server (LmConnection *connection, const gchar *server)
@@ -1384,13 +1518,47 @@ lm_connection_set_server (LmConnection *connection, const gchar *server)
 		return;
 	}
 	
-	if (connection->server) {
-		g_free (connection->server);
-	}
-	
+	g_free (connection->server);
 	connection->server = g_strdup (server);
 }
 
+/**
+ * lm_connection_get_jid:
+ * @connection: an #LmConnection
+ * 
+ * Fetches the jid set for @connection is using. 
+ * 
+ * Return value: the jid
+ **/
+const gchar *
+lm_connection_get_jid (LmConnection *connection)
+{
+	g_return_val_if_fail (connection != NULL, NULL);
+
+	return connection->jid;
+}
+
+/**
+ * lm_connection_set_jid:
+ * @connection: an #LmConnection
+ * @jid: JID to be used for @connection
+ * 
+ * Sets the JID to be used for @connection.
+ **/
+void 
+lm_connection_set_jid (LmConnection *connection, const gchar *jid)
+{
+	g_return_if_fail (connection != NULL);
+
+	if (lm_connection_is_open (connection)) {
+		g_warning ("Can't change JID while connected");
+		return;
+	}
+
+	g_free (connection->jid);
+	connection->jid = g_strdup (jid);
+}
+
 /**
  * lm_connection_get_port:
  * @connection: an #LmConnection
@@ -1466,7 +1634,6 @@ lm_connection_set_ssl (LmConnection *connection, LmSSL *ssl)
 	}
 }
 
-#if 0
 /**
  * lm_connection_get_proxy: 
  * @connection: an #LmConnection
@@ -1488,9 +1655,9 @@ lm_connection_get_proxy (LmConnection *connection)
  * @connection: an #LmConnection
  * @proxy: an #LmProxy
  *
- * Sets the proxy to use for this connection.
+ * Sets the proxy to use for this connection. To unset pass a proxy of type 
+ * #LM_PROXY_TYPE_NONE to this function.
  * 
- * Return value: The proxy or %NULL if no proxy is used. Notice that @connection can't be open while doing this.
  **/
 void
 lm_connection_set_proxy (LmConnection *connection, LmProxy *proxy)
@@ -1502,14 +1669,16 @@ lm_connection_set_proxy (LmConnection *connection, LmProxy *proxy)
 		g_warning ("Can't change server proxy while connected");
 		return;
 	}
-	
+
 	if (connection->proxy) {
 		lm_proxy_unref (connection->proxy);
+		connection->proxy = NULL;
 	}
 
-	connection->proxy = lm_proxy_ref (proxy);
+	if (lm_proxy_get_type (proxy) != LM_PROXY_TYPE_NONE) {
+		connection->proxy = lm_proxy_ref (proxy);
+	}
 }
-#endif
 
 /**
  * lm_connection_send: 
@@ -1619,22 +1788,23 @@ lm_connection_send_with_reply_and_block (LmConnection  *connection,
 		const gchar *m_id;
 		gint         n;
 
-		g_main_context_iteration (NULL, TRUE);
+		g_main_context_iteration (connection->context, TRUE);
 	
-		if (lm_queue_is_empty (connection->incoming_messages)) {
+		if (g_queue_is_empty (connection->incoming_messages)) {
 			continue;
 		}
 
-		for (n = 0; n < connection->incoming_messages->length; n++) {
+		for (n = 0; n < g_queue_get_length (connection->incoming_messages); n++) {
 			LmMessage *m;
 
-			m = lm_queue_peek_nth (connection->incoming_messages, n);
+			m = (LmMessage *) g_queue_peek_nth (connection->incoming_messages, n);
 
 			m_id = lm_message_node_get_attribute (m->node, "id");
 			
 			if (m_id && strcmp (m_id, id) == 0) {
 				reply = m;
-				lm_queue_remove_nth (connection->incoming_messages, n);
+				g_queue_pop_nth (connection->incoming_messages,
+						 n);
 				break;
 			}
 		}
@@ -1642,7 +1812,7 @@ lm_connection_send_with_reply_and_block (LmConnection  *connection,
 
 	g_free (id);
 	connection->incoming_source = connection_create_source (connection);
-	g_source_attach (connection->incoming_source, NULL);
+	g_source_attach (connection->incoming_source, connection->context);
 
 	return reply;
 }
@@ -1775,7 +1945,7 @@ LmConnectionState
 lm_connection_get_state (LmConnection *connection)
 {
 	g_return_val_if_fail (connection != NULL, 
-			      LM_CONNECTION_STATE_DISCONNECTED);
+			      LM_CONNECTION_STATE_CLOSED);
 
 	return connection->state;
 }
