@@ -1,7 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 Mikael Hallendal <micke@imendio.com>
- * Copyright (C) 2003 CodeFactory AB. 
+ * Copyright (C) 2003 Imendio HB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License as
@@ -33,7 +32,8 @@
 typedef enum {
         LM_ERROR_CONNECTION_NOT_OPEN,
         LM_ERROR_CONNECTION_OPEN,
-        LM_ERROR_AUTH_FAILED
+        LM_ERROR_AUTH_FAILED,
+	LM_ERROR_CONNECTION_FAILED
 } LmError;
 
 GQuark lm_error_quark (void) G_GNUC_CONST;
