@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 Mikael Hallendal <micke@imendio.com>
+ * Copyright (C) 2003 Imendio HB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License as
@@ -62,10 +62,7 @@ void             _lm_message_node_add_child_node    (LmMessageNode    *node,
 LmMessageNode *  _lm_message_node_new               (const gchar      *name);
 void             _lm_debug_init                     (void);
 
-gboolean         _lm_proxy_negotiate                (LmProxy          *proxy,
-						     gint              fd,
-						     const gchar      *server,
-						     guint             port);
+
 gboolean         _lm_proxy_connect_cb               (GIOChannel *source,
                                                      GIOCondition condition,
                                                      gpointer data);
